# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
# Copyright (c) 2013 Canonical Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import multiprocessing
import os
import platform
import sys

from oslo.config import cfg

from nova.openstack.common.gettextutils import _   # noqa
from nova.openstack.common import jsonutils
from nova.openstack.common import log as logging


CONF = cfg.CONF

log = logging.getLogger(__name__)


class HostOps(object):
    def __init__(self):
        self._stats = None

    def get_host_stats(self, refresh=False):
        if refresh or not self._stats:
            self._update_stats()
        return self._stats

    def get_compute_stats(self):
        # Updating host information
        disk = self. _get_disk_info()
        memory = self._get_host_memory()

        dic = {'vcpus': 1,
               'local_gb': disk['total'],
               'vcpus_used': 0,
               'memory_mb': memory['total'],
               'memory_mb_used': memory['total'] - memory['free'],
               'local_gb_used': disk['total'] - disk['free'],
               'hypervisor_type': 'lxc',
               'hypervisor_version': '1.0',
               'hypervisor_hostname': platform.node(),
               'cpu_info': '?',
               'supported_instances': jsonutils.dumps([
                        ('i686', 'lxc', 'lxc'),
                        ('x86_64', 'lxc', 'lxc')]),
        }
        return dic

    def _update_stats(self):
        memory = self._get_host_memory()
        disk = self._get_disk_info()
        cpu = self._get_cpu_info()

        data = {}
        data["vpcus"] = 1
        data["cpu_info"] = \
                {'vendor': cpu['vendor'],
                 'model': cpu['features'],
                 'topology': {'cores': 1,
                              'socket': multiprocessing.cpu_count(),
                              'threads': 1}
                }
        data["disk_total"] = disk['total']
        data["disk_available"] = disk["free"]
        data["disk_used"] = data["disk_total"] - data["disk_available"]
        data["memory_mb"] = memory["total"]
        data["host_memory_total"] = memory["total"]
        data["host_memory_free"] = memory["free"]
        data['hypverisor_type'] = 'lxc'
        data['hypervisor_version'] = '1.0'
        data['hypervisor_hostname'] = platform.node()
        data['supported_instances'] = [('i686', 'lxc', 'lxc'),
                                       ('x86_64', 'lxc', 'lxc')]
        self._stats = data
        return data

    def _get_host_memory(self):
        if sys.platform.upper() not in ['LINUX2', 'LINUX3']:
            return 1

        data = {}
        for line in open('/proc/meminfo'):
            if ':' in line:
                key, value = line.split(":", 1)
                if key in ["MemTotal", "MemFree", "Buffers", "Cached"]:
                    data[key] = int(value.split()[0])
        return{'total': data['MemTotal'],
               'free': (data["MemFree"] +
                        data["Buffers"] + data["Cached"])}

    def _get_disk_info(self):
        hddinfo = os.statvfs(CONF.instances_path)
        total = hddinfo.f_frsize * hddinfo.f_blocks
        free = hddinfo.f_frsize * hddinfo.f_bavail
        return {'total': total, 'free': free}

    def _get_cpu_info(self):
        cpuinfo = dict()
        cpuinfo['features'] = list()
        if sys.platform.upper() not in ['LINUX2', 'LINUX3']:
            return 0

        f = open('/proc/cpuinfo', 'r')
        for line in f:
            if line.strip():
                    if line.rstrip('\n').startswith('vendor_id'):
                        cpuinfo['vendor'] = line.rstrip('\n').split(':')[1]
                    if line.rstrip('\n').startswith('model_name'):
                        cpuinfo['model'] = line.rstrip('\n').split(':')[1]
                    if line.rstrip('\n').startswith('flags'):
                        cpuinfo['features'] = line.rstrip('\n').split(':')[1]

        cpuinfo['features'] = list(set(cpuinfo['features']))
        return cpuinfo
