# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
# Copyright (c) 2013 Canonical Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
lxc hypervisor drvier
"""

import os

from oslo.config import cfg

from nova.compute import power_state
from nova.openstack.common.gettextutils import _  # noqa
from nova.openstack.common import log as logging
from nova.virt import driver
from nova.virt.lxc import hostops
from nova.virt.lxc import vmops
from nova.virt import virtapi


lxc_opts = [
    cfg.StrOpt('lxc_path',
                default='/var/lib/lxc',
                help='Default lxc configuration path'),
    cfg.StrOpt('cgroups_path',
               default='/sys/fs/cgroup',
               help='Default cgroups path'),
    cfg.StrOpt('lxc_config_template',
              default='$pybasedir/nova/virt/lxc/config.tmpl',
              help='Template file for lxc configuration file'),
    cfg.StrOpt('lxc_fstab_template',
              default='$pybasedir/nova/virt/lxc/fstab.tmpl',
              help='Template file for lxc fstab file'),
    ]

CONF = cfg.CONF
CONF.register_opts(lxc_opts)
CONF.import_opt('host', 'nova.netconf')

LOG = logging.getLogger(__name__)


class LXCDriver(driver.ComputeDriver):
    capabilities = {
        "has_imagecache": False,
        "supports_recreate": False,
        }

    """lxc hypervisor driver."""

    def __init__(self, virtapi, read_only=False):
        super(LXCDriver, self).__init__(virtapi)
        self.hostops = hostops.HostOps()
        self.vmops = vmops.VMOps()

    def init_host(self, host):
        if not os.path.exists("/usr/bin/lxc-version"):
            LOG.error(_('LXC is not installed'))

    def list_instances(self):
        return self.vmops.list_instances()

    def plug_vifs(self, instance, network_info):
        """Plug VIFs into networks."""
        pass

    def unplug_vifs(self, instance, network_info):
        """Unplug VIFs from networks."""
        pass

    def spawn(self, context, instance, image_meta, injected_files,
              admin_password, network_info=None, block_device_info=None):
        self.vmops.spawn(context, instance, image_meta, injected_files,
            admin_password, network_info, block_device_info)

    def reboot(self, context, instance, network_info, reboot_type,
               block_device_info=None, bad_volumes_callback=None):
        pass

    def power_off(self, instance):
        self.vmops.power_off(instance)

    def power_on(self, context, instance, network_info, block_device_info):
        self.vmops.power_on(instance)

    def restore(self, instance):
        pass

    def pause(self, instance):
        pass

    def unpause(self, instance):
        pass

    def suspend(self, instance):
        pass

    def resume(self, instance, network_info, block_device_info=None):
        pass

    def destroy(self, instance, network_info, block_device_info=None,
                destroy_disks=True, context=None):
        self.vmops.destroy(instance, network_info, block_device_info,
                           destroy_disks, context)
        self.virtapi.instance_update(context.get_admin_context(),
            instance['uuid'], {'power_state': power_state.SHUTDOWN})

    def get_info(self, instance):
        state = self.vmops.container_exists(instance)
        if state:
            pstate = power_state.RUNNING
        elif state is False:
            pstate = power_state.SHUTDOWN
        else:
            pstate = power_state.NOSTATE
        return {'state': pstate,
                'max_mem': 0,
                'mem': 0,
                'num_cpu': 2,
                'cpu_time': 0}

    def get_console_output(self, instance):
        return self.vmops.get_console_output(instance)

    def get_available_resource(self, nodename):
        return self.hostops.get_compute_stats()

    def get_host_stats(self, refresh=False):
        return self.hostops.get_host_stats(refresh)
